//
//  Match.swift
//  Amalgamation
//
//  Created by Charles Magahern on 10/8/17.
//

import Foundation

struct Match : Codable
{
    enum State : String, Codable
    {
        case unknown
        case pending
        case open
        case complete
    }
    
    var id:                 Int     = 0
    var state:              State   = .unknown
    var tournamentID:       Int     = 0
    var dateCreated:        Date    = Date()
    var location:           String?
    var player1ID:          Int?
    var player2ID:          Int?
    var winnerID:           Int?
    var loserID:            Int?
    var round:              Int     = 0
    var dateStarted:        Date?
    var scoresCSV:          String?
    
    var player1Score:       Int?
    {
        get
        {
            return self._scores.0
        }
    }
    
    var player2Score:       Int?
    {
        get
        {
            return self._scores.1
        }
    }
    
    internal var _scores:   (Int?, Int?)
    {
        get
        {
            var scores: (Int?, Int?)
            let components = self.scoresCSV?.components(separatedBy: "-")
            
            if let cmp1 = components?[safe: 0] {
                scores.0 = Int(cmp1)
            }
            
            if let cmp2 = components?[safe: 1] {
                scores.1 = Int(cmp2)
            }
            
            return scores
        }
    }
    
    // MARK: Coding Support
    
    enum CodingKeys : String, CodingKey
    {
        case id
        case state
        case tournamentID = "tournament_id"
        case dateCreated = "created_at"
        case location
        case player1ID = "player1_id"
        case player2ID = "player2_id"
        case winnerID = "winner_id"
        case loserID = "loser_id"
        case round
        case dateStarted = "started_at"
        case scoresCSV = "scores_csv"
    }
}

struct TournamentMatch
{
    var match:              Match
    var player1Participant: Participant?
    var player2Participant: Participant?
    
    var winnerParticipant:  Participant?
    {
        get
        {
            var winner: Participant?
            
            let winnerID = self.match.winnerID
            if winnerID != 0 {
                if winnerID == self.player1Participant?.id {
                    winner = self.player1Participant
                } else if winnerID == self.player2Participant?.id {
                    winner = self.player2Participant
                }
            }
            
            return winner
        }
    }
}
