//
//  CollectionExtensions.swift
//  Amalgamation
//
//  Created by Charles Magahern on 1/28/18.
//

import Foundation

extension MutableCollection
{
    mutating func shuffle()
    {
        let c = count
        guard c > 1 else { return }
        
        for (firstUnshuffled, unshuffledCount) in zip(indices, stride(from: c, to: 1, by: -1)) {
            let d: Int = numericCast(arc4random_uniform(numericCast(unshuffledCount)))
            let i = index(firstUnshuffled, offsetBy: d)
            swapAt(firstUnshuffled, i)
        }
    }
}

extension Collection
{
    subscript(safe index: Index) -> Element?
    {
        return indices.contains(index) ? self[index] : nil
    }
    
    func shuffled() -> [Element]
    {
        var result = Array(self)
        result.shuffle()
        return result
    }
}
