//
//  FibonacciSphere.swift
//  Amalgamation
//
//  Created by Charles Magahern on 2/24/18.
//

import Foundation
import simd

class FibonacciSphere
{
    let samples:    UInt
    let randomize:  Bool
    
    init(samples: UInt, randomize: Bool)
    {
        self.samples = samples
        self.randomize = randomize
    }
    
    func generatePoints() -> [float3]
    {
        let rnd = self.randomize ? MathUtil.frand() * Float(self.samples) : 1.0
        var points: [float3] = []
        let offset = 2.0 / Float(self.samples)
        let increment = Float.pi * (3.0 - sqrtf(5.0))
        
        for i in 0 ..< self.samples {
            let y = ((Float(i) * offset) - 1.0) + (offset / 2.0)
            let r = sqrtf(1.0 - powf(y, 2.0))
            let φ = Float((i + UInt(rnd)) % self.samples) * increment
            
            let x = cosf(φ) * r
            let z = sinf(φ) * r
            
            points.append(float3(x, y, z))
        }
        
        return points
    }
}
