//
//  MathUtil.swift
//  Amalgamation
//
//  Created by Charles Magahern on 3/27/18.
//

import Foundation
import simd

struct MathUtil
{
    static func frand() -> Float
    {
        return Float(arc4random()) / Float(UInt32.max)
    }
    
    static func computeLookAtMatrix(eye: simd_float3, center: simd_float3, up: simd_float3) -> simd_float3x3
    {
        let zAxis = simd_normalize(eye - center)
        let xAxis = simd_normalize(simd_cross(up, zAxis))
        let yAxis = simd_cross(zAxis, xAxis)
        
        return simd_float3x3(xAxis, yAxis, zAxis)
    }
    
    static func matrix3ToMatrix4(_ m: simd_float3x3) -> simd_float4x4
    {
        let p = simd_make_float4(m.columns.0, 0.0)
        let q = simd_make_float4(m.columns.1, 0.0)
        let r = simd_make_float4(m.columns.2, 0.0)
        let s = simd_make_float4(0.0, 0.0, 0.0, 1.0)
        
        return simd_float4x4(p, q, r, s)
    }
    
    static func shortestArc(pt1: float3, pt2: float3) -> simd_quatf
    {
        let a = cross(pt1, pt2)
        var quat = float4()
        quat.x = a.x
        quat.y = a.y
        quat.z = a.z
        quat.w = sqrtf(powf(length(pt1), 2.0) * powf(length(pt2), 2.0)) + dot(pt1, pt2)
        
        return simd_quatf(vector: normalize(quat))
    }
    
    static func arcLength(position: simd_float3,
                          orientation: simd_quatf,
                          arcQuaternion: simd_quatf,
                          sampleCount: UInt = 256) -> Float
    {
        var length: Float = 0.0
        
        let nextPointBlock = { (t: Float) -> float3 in
            let sq = simd_slerp(orientation, arcQuaternion, t)
            return position * simd_matrix3x3(sq)
        }
        
        for i in 0 ..< sampleCount {
            let curPt = nextPointBlock(Float(i) / Float(sampleCount))
            let nextPt = nextPointBlock(Float(i + 1) / Float(sampleCount))
            length += simd_distance(curPt, nextPt)
        }
        
        return length
    }
}
