//
//  ServerOperation.swift
//  Amalgamation
//
//  Created by Charles Magahern on 9/22/18.
//

import Foundation

class ServerOperation : Operation
{
    var baseURL:             URL
    var session:             URLSession
    internal(set) var error: Error?
    
    init(baseURL: URL, session: URLSession)
    {
        self.baseURL = baseURL
        self.session = session
    }
    
    func fetchData(_ url: URL) -> Data?
    {
        var fetchedData: Data?
        
        let semaphore = Semaphore(value: 0)
        let task = self.session.dataTask(with: url, completionHandler: { (data: Data?, response: URLResponse?, error: Error?) -> Void in
            fetchedData = data
            self.error = error
            
            semaphore.signal()
        })
        task.resume()
        
        semaphore.wait()
        return fetchedData
    }
    
    func fetchResponse(_ request: URLRequest) -> (Data?, URLResponse?)
    {
        var fetchedData: Data?
        var fetchedResponse: URLResponse?
        
        let semaphore = Semaphore(value: 0)
        let task = self.session.dataTask(with: request, completionHandler: { (data: Data?, response: URLResponse?, error: Error?) in
            fetchedResponse = response
            fetchedData = data
            self.error = error
            
            semaphore.signal()
        })
        task.resume()
        
        semaphore.wait()
        return (fetchedData, fetchedResponse)
    }
}
