//
//  Theme.swift
//  Amalgamation
//
//  Created by Charles Magahern on 1/15/18.
//

import UIKit

typealias TextAttributes = [NSAttributedStringKey : Any]

struct ColorPalette
{
    var primaryColor:   UIColor = UIColor.black
    var secondaryColor: UIColor = UIColor.white
    var colors:         [UIColor] = [UIColor.black, UIColor.white] // should be arranged from darkest to lightest
}

struct Theme
{
    var headerTextAttributes:    TextAttributes = [:]
    var bodyTextAttributes:      TextAttributes = [:]
    var bodySubheaderAttributes: TextAttributes = [:]
    var statusTextAttributes:    TextAttributes = [:]
    var accessoryTextAttributes: TextAttributes = [:]
    
    var backgroundColor:         UIColor = UIColor()
    var foregroundColor:         UIColor = UIColor()
    var colorPalette:            ColorPalette = ColorPalette()
    
    var transitionDuration:      TimeInterval = 0.0
    
    static let mainTheme: Theme =
    {
        let headerFont = UIFont(name: "Karla-Regular", size: 24.0)!
        let bodyFont = UIFont(name: "Karla-Regular", size: 18.0)!
        let statusFont = UIFont(name: "Exan-Regular", size: 24.0)!
        let accessoryFont = UIFont(name: "Karla-Regular", size: 16.0)!
        
        let backgroundColor = UIColor(red: 0.07, green: 0.08, blue: 0.05, alpha: 1.00)
        let foregroundColor = UIColor(white: 0.8, alpha: 1.0)
        let metadataLabelColor = UIColor(white: 0.6, alpha: 1.0)
        
        let bodyParagraphStyle = NSMutableParagraphStyle()
        bodyParagraphStyle.lineSpacing = 8.0
        
        let headerAttributes: TextAttributes = [
            NSAttributedStringKey.font : headerFont,
            NSAttributedStringKey.foregroundColor : foregroundColor,
            NSAttributedStringKey.backgroundColor : backgroundColor,
            NSAttributedStringKey.kern : (50.0 * headerFont.pointSize / 1000.0)
        ]
        
        let bodyAttributes: TextAttributes = [
            NSAttributedStringKey.font : bodyFont,
            NSAttributedStringKey.foregroundColor : foregroundColor,
            NSAttributedStringKey.backgroundColor : backgroundColor,
            NSAttributedStringKey.paragraphStyle : bodyParagraphStyle
        ]
        
        let bodySubheaderAttributes: TextAttributes = [
            NSAttributedStringKey.font : bodyFont,
            NSAttributedStringKey.foregroundColor : metadataLabelColor,
            NSAttributedStringKey.backgroundColor : backgroundColor,
            NSAttributedStringKey.paragraphStyle : bodyParagraphStyle
        ]
        
        let statusAttributes: TextAttributes = [
            NSAttributedStringKey.font : statusFont,
            NSAttributedStringKey.foregroundColor : foregroundColor,
            NSAttributedStringKey.backgroundColor : backgroundColor
        ]
        
        let accessoryAttributes: TextAttributes = [
            NSAttributedStringKey.font : accessoryFont,
            NSAttributedStringKey.foregroundColor : foregroundColor,
            NSAttributedStringKey.backgroundColor : backgroundColor
        ]
        
        let paletteColors = [
            UIColor(rgbValue: 0x2a333b),
            UIColor(rgbValue: 0x373d3b),
            UIColor(rgbValue: 0x43473a),
            UIColor(rgbValue: 0x50513a),
            UIColor(rgbValue: 0x5c5a39),
            UIColor(rgbValue: 0x696439),
            UIColor(rgbValue: 0x756e39),
            UIColor(rgbValue: 0x827838),
            UIColor(rgbValue: 0x8e8238),
            UIColor(rgbValue: 0x9b8c37),
            UIColor(rgbValue: 0xa79537),
            UIColor(rgbValue: 0xb49f36),
            UIColor(rgbValue: 0xc0a936)
        ]
        let colorPalette = ColorPalette(primaryColor: paletteColors.last!,
                                        secondaryColor: paletteColors.first!,
                                        colors: paletteColors)
        
        return Theme(headerTextAttributes: headerAttributes,
                     bodyTextAttributes: bodyAttributes,
                     bodySubheaderAttributes: bodySubheaderAttributes,
                     statusTextAttributes: statusAttributes,
                     accessoryTextAttributes: accessoryAttributes,
                     backgroundColor: backgroundColor,
                     foregroundColor: foregroundColor,
                     colorPalette: colorPalette,
                     transitionDuration: 0.7)
    }()
}
