//
//  LoadingBarView.swift
//  Amalgamation
//
//  Created by Charles Magahern on 6/3/18.
//

import UIKit

class LoadingBarView : UIView
{
    private let _rectLayer:   CAShapeLayer = CAShapeLayer()
    private var _isAnimating: Bool = false
    
    override init(frame: CGRect)
    {
        super.init(frame: frame)
        
        _rectLayer.fillColor = UIColor.lightGray.cgColor
        self.layer.addSublayer(_rectLayer)
        self.layer.masksToBounds = true
    }
    
    required init?(coder aDecoder: NSCoder)
    {
        fatalError("unsupported")
    }
    
    override func layoutSubviews()
    {
        super.layoutSubviews()
        
        let bounds = self.bounds
        _rectLayer.frame = bounds
        
        var rect = bounds
        rect.size.width /= 2.0
        _rectLayer.path = CGPath(rect: rect, transform: nil)
        
        if _isAnimating {
            stopAnimating()
            startAnimating()
        }
    }
    
    // MARK: API
    
    func startAnimating()
    {
        if !_isAnimating {
            let anim = CABasicAnimation(keyPath: "transform.translation.x")
            anim.fromValue = -self.bounds.size.width / 2.0
            anim.toValue = self.bounds.size.width
            anim.repeatCount = .greatestFiniteMagnitude
            anim.duration = 1.0
            anim.timingFunction = CAMediaTimingFunction(name: kCAMediaTimingFunctionEaseInEaseOut)
            _rectLayer.add(anim, forKey: nil)
            
            _isAnimating = true
        }
    }
    
    func stopAnimating()
    {
        _rectLayer.removeAllAnimations()
        _isAnimating = false
    }
}
