//
//  TournamentBracketAnnotationsView.swift
//  Amalgamation
//
//  Created by Charles Magahern on 1/31/18.
//

import Foundation
import UIKit

class TournamentBracketAnnotationsView : UIView, Transitionable
{
    override init(frame: CGRect)
    {
        super.init(frame: frame)
        self.backgroundColor = UIColor.clear
    }
    
    required init?(coder aDecoder: NSCoder)
    {
        fatalError("unsupported")
    }
    
    // MARK: Accessors
    
    var roundsToMatchViews: [Int : [TournamentMatchView]] = [:]
    {
        didSet
        {
            self.setNeedsDisplay()
        }
    }
    
    // MARK: UIView
    
    override func draw(_ rect: CGRect)
    {
        super.draw(rect)
        
        let path = CGMutablePath()
        let lineWidth: CGFloat = 1.0
        
        // XXX: this sort function is copied in a bunch of places.
        let sortedRounds = self.roundsToMatchViews.sorted { abs($0.0) < abs($1.0) }
        
        for (idx, roundNumberMatchViewsPair) in sortedRounds.enumerated() {
            guard let nextRoundMatchViews = sortedRounds[safe: idx + 1]?.value else { continue }
            let roundMatchViews = roundNumberMatchViewsPair.value

            for roundMatchView in roundMatchViews {
                guard let match = roundMatchView.match else { continue }
                guard let winner = match.winnerParticipant else { continue }
                
                var nextRoundCorrespondingMatchView: TournamentMatchView?
                for nextRoundMatchView in nextRoundMatchViews {
                    guard let player1 = nextRoundMatchView.match?.player1Participant else { continue }
                    guard let player2 = nextRoundMatchView.match?.player2Participant else { continue }
                    
                    if (winner == player1 || winner == player2) {
                        nextRoundCorrespondingMatchView = nextRoundMatchView
                        break
                    }
                }
                
                if let nextRoundCorrespondingMatchView = nextRoundCorrespondingMatchView {
                    let curRoundMatchViewFrame = roundMatchView.frame
                    let nextRoundMatchViewFrame = nextRoundCorrespondingMatchView.frame
                    let matchViewsHMargin = nextRoundMatchViewFrame.minX - curRoundMatchViewFrame.maxX
                    
                    let leftHRect = CGRect(
                        x: curRoundMatchViewFrame.maxX + 1.0,
                        y: rint(curRoundMatchViewFrame.origin.y + (curRoundMatchViewFrame.size.height / 2.0) - (lineWidth / 2.0)),
                        width: rint(matchViewsHMargin / 2.0 - lineWidth),
                        height: lineWidth
                    )
                    
                    let nextRoundMidY = rint(nextRoundMatchViewFrame.origin.y + (nextRoundMatchViewFrame.size.height / 2.0) - (lineWidth / 2.0))
                    let midVRectOriginY = min(nextRoundMidY, leftHRect.origin.y)
                    let midVRect = CGRect(
                        x: leftHRect.maxX,
                        y: midVRectOriginY,
                        width: lineWidth,
                        height: abs(leftHRect.origin.y - nextRoundMidY) + lineWidth
                    )
                    
                    let rightHRectOriginX = midVRect.maxX
                    let rightHRect = CGRect(
                        x: rightHRectOriginX,
                        y: nextRoundMidY,
                        width: nextRoundMatchViewFrame.minX - 1.0 - rightHRectOriginX,
                        height: lineWidth
                    )
                    
                    path.addRect(leftHRect)
                    path.addRect(midVRect)
                    path.addRect(rightHRect)
                }
            }
        }
        
        UIColor(white: 0.5, alpha: 1.0).setFill()
        let bezier = UIBezierPath(cgPath: path)
        bezier.fill()
    }
    
    // MARK: Transitionable
    
    func transitionToActive(transition: Transition, completion: @escaping (Bool) -> Void)
    {
        self.alpha = 0.0
        UIView.animate(withDuration: transition.duration, delay: transition.delay, options: transition.options, animations: {
            self.alpha = 1.0
        }, completion: completion)
    }
    
    func transitionToInactive(transition: Transition, completion: @escaping (Bool) -> Void)
    {
        self.alpha = 1.0
        UIView.animate(withDuration: transition.duration, delay: transition.delay, options: transition.options, animations: {
            self.alpha = 0.0
        }, completion: completion)
    }
}
