//
//  TournamentStatsView.swift
//  Amalgamation
//
//  Created by Charles Magahern on 1/15/18.
//

import UIKit

class TournamentStatsView : UIView
{
    private var _statsTextView:    UITextView = UITextView()
    
    override init(frame: CGRect)
    {
        super.init(frame: frame)
        
        _statsTextView.isSelectable = false
        _statsTextView.isScrollEnabled = false
        _statsTextView.textAlignment = .natural
        _statsTextView.textContainer.lineFragmentPadding = 0.0
        _statsTextView.textContainerInset = .zero
        self.addSubview(_statsTextView)
    }
    
    required init?(coder aDecoder: NSCoder)
    {
        fatalError("unsupported")
    }
    
    override func layoutSubviews()
    {
        super.layoutSubviews()
        
        let bounds = self.bounds
        _statsTextView.frame = bounds
    }
    
    override func sizeThatFits(_ size: CGSize) -> CGSize
    {
        return _statsTextView.sizeThatFits(size)
    }
    
    // MARK: API
    
    var tournament: Tournament?
    {
        didSet
        {
            if let tournament = tournament {
                let theme = Theme.mainTheme
                let valueAttrs = theme.bodyTextAttributes
                let keyAttrs = theme.bodySubheaderAttributes
                let dateFormatter = DateFormatter()
                dateFormatter.dateStyle = .full
                
                let metadataPairs: [(String, String)] = [
                    ("TOURNAMENT_TYPE_LABEL", tournament.type.localizedDescription),
                    ("TOURNAMENT_GAME_LABEL", tournament.gameName),
                    ("TOURNAMENT_DATE_LABEL", dateFormatter.string(from: tournament.dateStarted ?? Date())),
                    ("TOURNAMENT_PLAYERS_COUNT_LABEL", "\(tournament.participantsCount)"),
                    ("TOURNAMENT_MATCHES_COUNT_LABEL", "\(tournament.matches.count)")
                ]
                
                let statsAttributedText = NSMutableAttributedString()
                for (idx, pair) in metadataPairs.enumerated() {
                    let (labelLocalizationKey, value) = pair
                    let locale = Locale.current
                    let uppercaseValue = value.uppercased(with: locale)
                    let localizedLabel = NSLocalizedString(labelLocalizationKey, comment: "")
                    let uppercaseLocalizedLabelWithDelimiter = "\(localizedLabel.uppercased(with: locale)): "
                    let pairString = "\(uppercaseLocalizedLabelWithDelimiter)\(uppercaseValue)"
                    let pairAttrString = NSMutableAttributedString(string: pairString)
                    
                    pairAttrString.addAttributes(keyAttrs, range: NSRange(pairString.range(of: uppercaseLocalizedLabelWithDelimiter)!, in: pairString))
                    pairAttrString.addAttributes(valueAttrs, range: NSRange(pairString.range(of: uppercaseValue)!, in: pairString))
                    statsAttributedText.append(pairAttrString)
                    
                    if idx != metadataPairs.count - 1 {
                        statsAttributedText.append(NSAttributedString(string: "\n"))
                    }
                }
                
                _statsTextView.attributedText = statsAttributedText
            } else {
                _statsTextView.attributedText = nil
            }
            
            self.setNeedsLayout()
        }
    }
}
